//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
//
//
//
// Heavy Gear 2 Launcher that enables the playback of music using .mp3 files.
// (also supports .wav files)
//
// Music must be located in a subfolder called \Music\ in the main game dir.
// Music files must be named in the format: Track1.mp3, Track2.mp3, etc...
// Up to 18 tracks can be played. Note that not all tracks need to be present 
// for the music to play. So you can omit tracks you don't like.
//
// MinGW Command used to compile:
// gcc HG2Launcher.c -lpsapi -lwinmm -o HG2Launcher.exe
//
// If compiling with Visual C++ make sure "Multi-Byte Character Set" 
// is enabled in Project->Properties.
//
// Code example used for getting the module names:
// https://docs.microsoft.com/en-us/windows/desktop/psapi/enumerating-all-modules-for-a-process
//

#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <psapi.h>
#include <tlhelp32.h>
#include <string.h>
#include <time.h>

// To ensure correct resolution of symbols, add Psapi.lib to TARGETLIBS
// and compile with -DPSAPI_VERSION=1
#define PSAPI_VERSION 1
#pragma comment(lib, "psapi.lib")
#pragma comment(lib, "winmm.lib")

const char g_szClassName[] = "myWindowClass";

int playing = 0;
int mp3done = 0;

typedef int bool_C;
#define true 1
#define false 0
bool_C musictoggle;

char *fn;
char *input;
char processName[] = "Heavy Gear 2.exe";
char toFind1[] = "HG2SIM.DLL";
char toFind2[] = "SHELL.DLL";

HANDLE player = NULL;
HANDLE WindowMsg = NULL;
HWND hwnd;

// Mp3 player thread using MCI.
//-----------------------------
int player_main( void )
{

// HG2Launcher.exe volume control with ini file:

	int HG2music_vol = 500;
	char invalid_ini_file = 0;

	FILE * fp;
	fp = fopen ("HG2Launcher.ini", "r");
			if (fopen == NULL){
			invalid_ini_file = 1;
			}
	fscanf(fp, "%d", &HG2music_vol);
	fclose(fp);
	if (invalid_ini_file = 1){
		FILE * fp;
		fp = fopen ("HG2Launcher.ini", "w+");
		fprintf(fp, "%d\n"
					"#\n"
					"# HG2Launcher.exe volume control.\n"
					"# Change the number to the desired volume level (0-1000).", HG2music_vol);
		fclose(fp);
		}
	if (HG2music_vol < 0) HG2music_vol = 0;
    if (HG2music_vol > 1000) HG2music_vol = 1000;

    char setvolume[100];
  	sprintf(setvolume, "setaudio hg2mp3 volume to %d", HG2music_vol);

//========
musicloop:

if (playing == 1 && musictoggle == 0){

	// Choose a track randomly.
	srand(time(NULL));
	int rndno = (rand() % 18 + 1); // Random track number (1-18).

	// Try playing .mp3 and revert to .wav on error.
	// We can use "type mpegvideo" for .wav files too. This enables volume control
	// which would not work if device type was set to "type waveaudio".
	// https://docs.microsoft.com/en-us/windows/desktop/multimedia/playing-a-device
	// https://docs.microsoft.com/en-us/windows/desktop/multimedia/multimedia-command-strings
	char trackno[100];
	sprintf(trackno, "open \".\\Music\\Track%d.mp3\" type mpegvideo alias hg2mp3", rndno);
	char trackno2[100];
	sprintf(trackno2, "open \".\\Music\\Track%d.wav\" type mpegvideo alias hg2mp3", rndno);

	if( (int) (mciSendStringA(trackno, NULL, 0, NULL) != 0)){
		mciSendStringA(trackno2, NULL, 0, NULL);
		printf( "\nTrack%d.mp3 not found! Trying .wav instead...", rndno );
	}
	printf( "\nTrack%d", rndno );

	//Note: Volume setting does not work for .wav files if device type is "waveaudio".
	mciSendStringA(setvolume, NULL, 0, NULL); //Volume 0-1000

	if( (int) (mciSendStringA("play hg2mp3 notify", NULL, 0, hwnd) != 0)){
		printf( " not found!\n", rndno );
		Sleep (1000); // Needs 1 second to generate a new random number.
		goto musicloop;
	}

	while (playing == 1 && mp3done == 0 && musictoggle == 0){
	Sleep (1000);
	}

	mciSendStringA("close hg2mp3", NULL, 0, NULL);
	mp3done = 0;
}

	Sleep (1000);

goto musicloop;
//=============

	return 0;
}

// MM_MCINOTIFY message handling to support "notify" command.
//-----------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch(msg)
    {
        case MM_MCINOTIFY:
        {
	        //if (msg==MM_MCINOTIFY && wParam==MCI_NOTIFY_SUCCESSFUL)
	        mp3done = 1;
        }
        break;

        case WM_CLOSE:
            DestroyWindow(hwnd);
        break;

        case WM_DESTROY:
            PostQuitMessage(0);
        break;

        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);

    }
    return 0;
}

// Needed for finding out if Heavy Gear 2.exe is no longer running.
//-----------------------------------------------------------------
int FindProcessId(char *processName)
{
	PROCESSENTRY32 processInfo;
	processInfo.dwSize = sizeof(processInfo);

	HANDLE processesSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if ( processesSnapshot == INVALID_HANDLE_VALUE )
		return 0;

	Process32First(processesSnapshot, &processInfo);
	if(_stricmp(_strupr(processInfo.szExeFile), processName)==0)
	{
		CloseHandle(processesSnapshot);
		return processInfo.th32ProcessID;
	}

	while ( Process32Next(processesSnapshot, &processInfo) )
	{
		if(_stricmp(_strupr(processInfo.szExeFile), processName)==0)
		{
			CloseHandle(processesSnapshot);
			return processInfo.th32ProcessID;
		}
	}
	
	CloseHandle(processesSnapshot);

	return 0;
}

//Adapted Microsoft example code for getting module names.
//--------------------------------------------------------
int PrintModules( DWORD processID )
{
    HMODULE hMods[1024];
    HANDLE hProcess;
    DWORD cbNeeded;
    unsigned int i;

    // Print the process identifier.

    //printf( "\nProcess ID: %u\n", processID );

    // Get a handle to the process.

    hProcess = OpenProcess( PROCESS_QUERY_INFORMATION |
                            PROCESS_VM_READ,
                            FALSE, processID );
    if (NULL == hProcess)
        return 1;

   // Get a list of all the modules in this process.

    if( EnumProcessModules(hProcess, hMods, sizeof(hMods), &cbNeeded))
    {
        for ( i = 0; i < (cbNeeded / sizeof(HMODULE)); i++ )
        {
            TCHAR szModName[MAX_PATH];

            // Get the full path to the module's file.

            if ( GetModuleFileNameEx( hProcess, hMods[i], szModName,
                                      sizeof(szModName) / sizeof(TCHAR)))
            {
                // Print the module name and handle value.

                //_tprintf( TEXT("\t%s (0x%08X)\n"), szModName, hMods[i] );
                
                // Print the module file name only omiting the path:
                input = szModName;
                (fn = strrchr(input, '\\')) ? ++fn : (fn = input);
                //printf("%s\n", fn);
                
				// If HG2SIM.DLL is found that means player is in mission and
				// music should play:
                if (strcmp(toFind1,fn) == 0)
				{
					if (playing == 0)
					{
					printf( "\nHG2SIM.DLL found! Playing music:" );
					playing = 1;
					}
				}
				
				// If SHELL.DLL is found player is in game menu and music is
				// stopped:
                if (strcmp(toFind2,fn) == 0)
				{
					if (playing == 1)
					{
					printf( "\nSHELL.DLL found! Music stopped." );
					playing = 0;
					musictoggle = 0;
					}
				}

            }
        }
    }
    
    // Release the handle to the process.

    CloseHandle( hProcess );

    return 0;
}

//The main console process.
//-------------------------
int main( void )
{
    DWORD aProcesses[1024]; 
    DWORD cbNeeded; 
    DWORD cProcesses;
    DWORD processID;
    unsigned int i;

	// Checks that program is not already running.
	CreateMutexA(NULL, TRUE, "HG2LauncherMutex");
	if (GetLastError() == ERROR_ALREADY_EXISTS)
		{
			return 0;
		}
	
	// Set up console window and start HG2:
	SetConsoleTitle( TEXT( "Heavy Gear 2 Launcher Status Window" ) );
	ShowWindow(GetConsoleWindow(), SW_SHOWNOACTIVATE);
	system("Color 3b");

	if( (int) ShellExecute(NULL, "open",
	             "Heavy Gear 2.exe",
	             NULL, NULL, SW_SHOWDEFAULT) <= 32)
	{
	     printf( "Heavy Gear 2.exe not found! Exiting...\n" );
	     Sleep (3000);
	     return 0;
	}
	else
	{
	    printf( "Heavy Gear 2 started." );
	}
	
	// Starting threads:
	player = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)player_main, NULL, 0, NULL);
	WindowMsg = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)WinMain, NULL, 0, NULL);
	
//===
loop:

	// Find out if Heavy Gear 2.exe is no longer running and if so then exit.

	processID = FindProcessId(processName);

	if ( processID == 0 ){
		printf( "\nShutting down HG2Launcher..." );
		goto end; //End loop.
		//========
	}
		else{
			printf( "." );
		}

    // Get the list of process identifiers.

    if ( !EnumProcesses( aProcesses, sizeof(aProcesses), &cbNeeded ) )
        return 1;

    // Calculate how many process identifiers were returned.

    cProcesses = cbNeeded / sizeof(DWORD);

    // Print the names of the modules for each process.

    for ( i = 0; i < cProcesses; i++ ){
        PrintModules( aProcesses[i] );
    }
    
    // Manual music toggle using Pause|Break key:
	if (GetAsyncKeyState(VK_SCROLL) && playing == 1){
	musictoggle = !musictoggle;
	printf( "\nMusic Toggle..." );
	}
    
    // Sleep a while...
	Sleep (1000);
	
goto loop;
//========
end:

	// Close hg2mp3 just in case...
	mciSendStringA("close hg2mp3", NULL, 0, NULL);

    return 0;
}

// Winmain thread needed to enable message handling.
//--------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
{

    WNDCLASSEX wc;
    
    MSG Msg;

    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = 0;
    wc.lpfnWndProc   = WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = g_szClassName;
    wc.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

    if(!RegisterClassEx(&wc)){
        MessageBox(NULL, "Window Registration Failed!", "Error!",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    hwnd = CreateWindowEx(
        WS_EX_CLIENTEDGE,
        g_szClassName,
        "HG2LauncherWindow",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, 240, 120,
        NULL, NULL, hInstance, NULL);

    if(hwnd == NULL){
        MessageBox(NULL, "Window Creation Failed!", "Error!",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    //ShowWindow(hwnd, nCmdShow);
    //UpdateWindow(hwnd);

    while(GetMessage(&Msg, NULL, 0, 0) > 0){
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }
    return Msg.wParam;

}
